# coding: utf-8
"""
    Snowflake API Integration API.

    The Snowflake API Integration API is a REST API that you can use to access, update, and perform certain actions on API Integration resource in a Snowflake database.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Contact: support@snowflake.com
    Generated by: https://openapi-generator.tech

    Do not edit this file manually.
"""

from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from typing import Union

from snowflake.core.api_integration._generated.models.api_hook import ApiHook

from pydantic import ConfigDict, StrictStr, field_validator

from typing import Any, ClassVar, Dict, List, Optional


class AzureHook(ApiHook):
    """A model object representing the AzureHook resource.

    Constructs an object of type AzureHook with the provided properties.

    Parameters
    __________
    api_provider : str

    azure_tenant_id : str
        The ID of the Office 365 tenant that all Azure API Management instances belong to.
    azure_ad_application_id : str
        The application ID of the Azure Active Directory application.
    api_key : str, optional
        An alphanumeric string that is used to identify API clients and control access to the API, also called a subscription key.
    """

    api_provider: StrictStr

    azure_tenant_id: StrictStr

    azure_ad_application_id: StrictStr

    api_key: Optional[StrictStr] = None

    __properties = ["type"]

    @field_validator('api_provider')
    def api_provider_validate_enum(cls, v):

        if v not in ('AZURE_API_MANAGEMENT', 'AZURE_PRIVATE_API_MANAGEMENT'):
            raise ValueError(
                "must validate the enum values ('AZURE_API_MANAGEMENT','AZURE_PRIVATE_API_MANAGEMENT')"
            )
        return v

    class Config:
        populate_by_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> AzureHook:
        """Create an instance of AzureHook from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""

        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update({})

        _dict = dict(
            self._iter(to_dict=True,
                       by_alias=True,
                       exclude=exclude_properties,
                       exclude_none=True))

        # set to None if api_key (nullable) is None
        if self.api_key is None:
            _dict['api_key'] = None

        _dict['type'] = ApiHook.get_child_model_discriminator_value(
            'AzureHook')

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> AzureHook:
        """Create an instance of AzureHook from a dict."""

        if obj is None:
            return None

        if type(obj) is not dict:
            return AzureHook.parse_obj(obj)

        _obj = AzureHook.parse_obj({
            "api_provider":
            obj.get("api_provider"),
            "azure_tenant_id":
            obj.get("azure_tenant_id"),
            "azure_ad_application_id":
            obj.get("azure_ad_application_id"),
            "api_key":
            obj.get("api_key"),
        })

        return _obj


from typing import Optional, List, Dict

from snowflake.core.api_integration._generated.models.api_hook import ApiHook


class AzureHookModel(ApiHook):

    def __init__(
        self,
        api_provider: str,
        azure_tenant_id: str,
        azure_ad_application_id: str,
        # optional properties
        api_key: Optional[str] = None,
    ):
        """A model object representing the AzureHook resource.

        Constructs an object of type AzureHook with the provided properties.

        Parameters
        __________
        api_provider : str

        azure_tenant_id : str
            The ID of the Office 365 tenant that all Azure API Management instances belong to.
        azure_ad_application_id : str
            The application ID of the Azure Active Directory application.
        api_key : str, optional
            An alphanumeric string that is used to identify API clients and control access to the API, also called a subscription key.
        """

        super().__init__()
        self.api_provider = api_provider
        self.azure_tenant_id = azure_tenant_id
        self.azure_ad_application_id = azure_ad_application_id
        self.api_key = api_key

    __properties = ["type"]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return AzureHook(
            api_provider=self.api_provider,
            azure_tenant_id=self.azure_tenant_id,
            azure_ad_application_id=self.azure_ad_application_id,
            api_key=self.api_key,
        )

    @classmethod
    def _from_model(cls, model) -> AzureHookModel:
        return AzureHookModel(
            api_provider=model.api_provider,
            azure_tenant_id=model.azure_tenant_id,
            azure_ad_application_id=model.azure_ad_application_id,
            api_key=model.api_key,
        )

    def to_dict(self):
        """Creates a dictionary of the properties from a AzureHook.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the AzureHook object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> AzureHookModel:
        """Creates an instance of AzureHook from a dict.

        This method constructs a AzureHook object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        AzureHook
            A AzureHook object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(AzureHook.from_dict(obj))


AzureHook._model_class = AzureHookModel
